package com.demo.insurance.to;

import com.demo.soa.esb.util.to.ModelBean;

/**
 * This class represents an insurance application. It was created specifically
 * for this demo project and defines the minimal number of fields necessary to
 * create a realistic example.
 * 
 * @author Leon Rubin
 */
public class Application extends ModelBean {
	private String number;
	private int status;
	private boolean authorizationFormAttached;
	private PersonalInfo applicantInfo;

	/**
	 * Gets the personal informaiton of the applicant.
	 * 
	 * @return personal informaiton of the applicant
	 */
	public PersonalInfo getApplicantInfo() {
		return applicantInfo;
	}

	/**
	 * Sets the personal informaiton of the applicant.
	 * 
	 * @param applicantInfo
	 *            personal informaiton of the applicant
	 */
	public void setApplicantInfo(PersonalInfo applicantInfo) {
		this.applicantInfo = applicantInfo;
	}

	/**
	 * Gets the application number.
	 * 
	 * @return application number
	 */
	public String getNumber() {
		return number;
	}

	/**
	 * Sets the application number.
	 * 
	 * @param number
	 *            application number
	 */
	public void setNumber(String number) {
		this.number = number;
	}

	/**
	 * Gets the application status. The status changes as the application moves
	 * through stages of its lifecycle (NEW, VALIDATED, DECLINED, APPROVED,
	 * etc). Valid status values are defined in the ApplicationStatus class. In
	 * the future (when there is no requirement to be backward compatible with
	 * JDK 1.4), the application status can be represented by an enumeration.
	 * 
	 * @return application status
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * Gets the application status as a string which is often useful for
	 * debugging and other purposes since the status is stored as integer. In
	 * the future (when there is no requirement to be backward compatible with
	 * JDK 1.4), the application status can be represented by an enumeration and
	 * this method may not be needed
	 * 
	 * @return application status name
	 */
	public String getStatusName() {
		return ApplicationStatus.getStatusName(status);
	}

	/**
	 * Sets the application status.
	 * 
	 * @param status
	 *            integer code representing the new application status. Use
	 *            constants from the ApplicationStatus class as parameters to
	 *            this method
	 */
	public void setStatus(int status) {
		this.status = status;
	}

	/**
	 * Indicates whether the application has an authorization form attached that
	 * authorizes the insurance company to search for applicant's medical
	 * information. This is common in insurance business and may be due to some
	 * HIPA regulations.
	 * 
	 * @return <code>true</code> if an authorization form is attached and
	 *         <code>false</code> otherwise
	 */
	public boolean isAuthorizationFormAttached() {
		return authorizationFormAttached;
	}

	/**
	 * Sets the authorization form indicator.
	 * 
	 * @param authorizationFormAttached
	 *            indicates whether the authorization form was attached
	 */
	public void setAuthorizationFormAttached(boolean authorizationFormAttached) {
		this.authorizationFormAttached = authorizationFormAttached;
	}

	/**
	 * Test the equality of this object and another object by comparing all
	 * individual address fields rather than the default behaviour of comparing
	 * references.
	 * 
	 * @return <code>true</code> if the objects are equal and
	 *         <code>false</code> otherwise
	 */
	public boolean equals(Object o) {
		boolean result = false;

		if (o instanceof Application) {
			Application application = (Application) o;

			if (number.equals(application.number)
					&& status == application.status
					&& authorizationFormAttached == application.authorizationFormAttached
					&& applicantInfo.equals(application.applicantInfo)) {
				result = true;
			}
		}

		return result;
	}

	/**
	 * Creates a hash code for this object by adding the hash codes of
	 * individual fields.
	 * 
	 * @return integer container the hash code for this object
	 */
	public int hashCode() {
		return hashCode(number) + hashCode(status)
				+ +hashCode(authorizationFormAttached)
				+ hashCode(applicantInfo);
	}

	/**
	 * Creates a string representation for this object by printing out the class
	 * name and individual fields to produce a more meaningful output than the
	 * default implementation.
	 * 
	 * @return String representation of this object
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();

		buffer.append(getClass().getName() + "{");
		buffer.append("\n\tnumber=" + number);
		buffer.append("\n\tstatus=" + status);
		buffer.append("\n\tauthorizationFormAttached="
				+ authorizationFormAttached);
		buffer.append(applicantInfo);
		buffer.append("}\n");

		return buffer.toString();
	}
}
