package com.demo.soa.esb.rulesengine;

import com.demo.soa.esb.util.exception.RulesEngineException;

/**
 * This interface declares methods that a typical rules engine is expected to
 * implement. It is an important part of a factory pattern that allows all
 * classes that use rules engine functionality remain independent of the
 * underlying implementation (Drools, etc). All calling classes must be coded to
 * this interface and obtain an implementation from the factory. Many Java based
 * rules engines use POJOs to represent facts. The engine maintains a set of all
 * facts that it is aware of in a collection called the working memory. New
 * facts are added to the working memory by calling the <code>insert</code>
 * method. Insertion of facts triggers rules making them eligable for execution.
 * For instance inserting an Employee object a reference to Address object with
 * city = Philadelphia, triggers the rule "If employee works in Philadelhia,
 * charge 4% city tax". The rules are evaluated upon insertion but not executed
 * until the <code>executeRules</code> method is called.
 * 
 * @author Leon Rubin
 */
public interface RulesEngine {
	/**
	 * Inserts a fact into the rules engine's working memory. Calling this
	 * method makes the rule engine aware of the fact. At the time of insertion,
	 * the engine evaluates all the rules that are relevant to this fact and
	 * prepares them for execution, however rules will not be executed until the
	 * <code>executeRules</code> is called
	 * 
	 * @param object
	 *            Java object that represents some fact
	 * @throws RulesEngineException
	 *             if a problem occurs during fact evaluation
	 */
	void insert(Object object) throws RulesEngineException;

	/**
	 * Executes rules defined in the rules engine that are eligible for
	 * execution based on the facts contained in the working memory.
	 * 
	 * @throws RulesEngineException
	 *             if a problem occurs during rules execution
	 */
	void executeRules() throws RulesEngineException;

	/**
	 * Releases all resources held by the rules engine.
	 * 
	 */
	void destroy();
}
