package com.demo.soa.esb.rulesengine.customj;

import java.util.List;

import com.demo.soa.esb.rulesengine.RulesEngine;
import com.demo.soa.esb.rulesengine.RulesEngineFactory;
import com.demo.soa.esb.rulesengine.drools.dao.WorkingMemoryDAOFactory;
import com.demo.soa.esb.util.customj.AbstractXQService;
import com.demo.soa.esb.util.customj.EndpointManager;
import com.demo.soa.esb.util.exception.DAOException;
import com.demo.soa.esb.util.exception.RulesEngineException;
import com.demo.soa.esb.util.exception.XMLException;
import com.demo.soa.esb.util.to.ModelBean;
import com.demo.soa.esb.xml.XMLParser;
import com.demo.soa.esb.xml.XMLParserFactory;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQPart;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceException;

/**
 * This Custom Java Service is the entry point of the rules engine. It shold be
 * should be notified of any new fact (object, document, or event that is
 * relevant to the system) by placing its XML representation on the entry
 * endpoint of this service. Every new fact will be converted back to its Java
 * object form and inserted into the working memory of the rules engine causing
 * rules to execute. Some rules will invoke other ESB services as needed.
 * 
 * @author Leon Rubin
 */
public class RBRService extends AbstractXQService {
	private RulesEngine rulesEngine;
	private XMLParser xmlParser;

	/**
	 * Constructor for a RBRService
	 */
	public RBRService() {
		super("RBRService");
	}

	/**
	 * Initializes the RBR service by obtaining references to the rules engine
	 * and the XML parser. Reads the persisted facts databases and populates the
	 * rules engine with any persisted data that may have survived the system
	 * crash or existed prior to the last system restart.
	 */
	public void init(XQInitContext initialContext) throws XQServiceException {
		super.init(initialContext);

		try {
			rulesEngine = RulesEngineFactory.getRulesEngine();
			xmlParser = XMLParserFactory.getXMLParser();
			EndpointManager.setXQEndpointManager(initialContext
					.getEndpointManager());

			List<ModelBean> objects = WorkingMemoryDAOFactory.getDAO()
					.getAllObjects();

			for (ModelBean object : objects) {
				System.out
						.println("Inserting persisted fact into working memory "
								+ object);
				rulesEngine.insert(object);
			}

			System.out.println("Done");
		} catch (RulesEngineException e) {
			e.printStackTrace();
			throw new XQServiceException("Failed to get a rules engine", e);
		} catch (XMLException e) {
			e.printStackTrace();
			throw new XQServiceException("Failed to get an XML parser", e);
		} catch (DAOException e) {
			e.printStackTrace();
			throw new XQServiceException("Failed to load saved facts", e);
		}
	}

	/**
	 * Processes the first part of every incoming message by parsing the XML
	 * part content, converting it into a Java object (fact), and inserting it
	 * into the working memory of the rules engine. The rules engine will
	 * evaluate the new fact and determine the rules that become eligible for
	 * execution. A call to <code>executeRule</code> method of the rules
	 * engine executes all eligible rules. That executes the Java code located
	 * in the consequence section of those rules. Some of that code simply
	 * changes the state of facts, other code may call various ESB services,
	 * etc.
	 */
	public void processPart(XQServiceContext ctx, XQEnvelope env,
			XQMessage msg, XQPart part, int partIndex)
			throws XQServiceException, XQMessageException {
		if (partIndex == 0) {
			String xml = part.getContent().toString();

			try {
				Object object = xmlParser.parse(xml);

				rulesEngine.insert(object);
				rulesEngine.executeRules();

				// This statement is necessary to compensate for the quirkiness
				// of the HSQL database. If the database is not closed properly,
				// the data may get corrupted. This issue does not occur with
				// MySQL.
				WorkingMemoryDAOFactory.getDAO().close();
			} catch (RulesEngineException e) {
				e.printStackTrace();
				throw new XQServiceException("Failed to execute rules", e);
			} catch (XMLException e) {
				e.printStackTrace();
				throw new XQServiceException("Failed to parse incoming XML", e);
			} catch (DAOException e) {
				e.printStackTrace();
				throw new XQServiceException("Failed to persist incoming fact",
						e);
			}
		}
	}

	/**
	 * Releases the resources held by this services (destroys the rules engine
	 * and the XML parser).
	 */
	public void destroy() {
		rulesEngine.destroy();
		xmlParser.destroy();
	}
}