package com.demo.soa.esb.rulesengine.drools;

import java.io.InputStreamReader;
import java.io.Reader;

import org.drools.RuleBase;
import org.drools.RuleBaseConfiguration;
import org.drools.RuleBaseFactory;
import org.drools.StatefulSession;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;

import com.demo.soa.esb.rulesengine.RulesEngine;
import com.demo.soa.esb.util.exception.RulesEngineException;

/**
 * This class is a Drools implementation of the rules engine interface. In
 * Drools, the rules are defined in .drl files using a special syntax. Each rule
 * consists of a condition and a consequence. The condition asserts one or more
 * facts and consequence contains Java code that will be executed if the
 * condition is true. DRL files are easy to read however Drools provide an
 * additional optional feature called DSL (Domain Specific Languages) that make
 * rules easy to read for business users. The developer can define a DSL file
 * that maps natural language statements into Drools rules. This class loads
 * both the DRL rules file and a DSL natural language preprocessor file and
 * prepares the rules engine for processing. Working memory is a temporary
 * object that is not persisted, however for this application it is important to
 * persist the working memory state so that the memory can be fully recovered in
 * the event of a system crash or a scheduled shutdown. Drools working memory
 * represented by the StatefulSession type allows to attache a listener to it
 * allowing to observe all fact events (insertions, updates, and deletions). A
 * special class called PersistingWorkingMemoryObserver implements the listener
 * interface and records every fact change in the database making the
 * information permanent. Typically, a MySQL database is used to persist the
 * memory state, however to keep the footprint of this demo small and setup
 * simple, this application uses a HSQL embedded database.
 * 
 * Javadoc comments for most of the methods are not provided because they will
 * be inherited from the implemented interface during generation.
 * 
 * @author Leon Rubin
 */
public class RulesEngineDroolsImpl implements RulesEngine {
	private StatefulSession session;
	private PersistingWorkingMemoryObserver persistingWorkingMemoryObserver;

	/**
	 * Creates the engine loading rules into memory and preparing working memory
	 * for user defined facts.
	 * 
	 * @throws RulesEngineException
	 *             if a problem occurs during rules engine initialization
	 */
	public RulesEngineDroolsImpl() throws RulesEngineException {
		try {
			RuleBase ruleBase = getRulesBase();

			session = ruleBase.newStatefulSession();

			persistingWorkingMemoryObserver = new PersistingWorkingMemoryObserver();
			session.addEventListener(persistingWorkingMemoryObserver);
		} catch (Exception e) {
			e.printStackTrace();
			throw new RulesEngineException(
					"Failed to initialize the rules engine", e);
		}
	}

	public void executeRules() throws RulesEngineException {
		session.fireAllRules();
	}

	public void insert(Object object) throws RulesEngineException {
		session.insert(object);
	}

	public void destroy() {
		session.dispose();
	}

	/**
	 * Loads the rules and the DLS mapping file from the classpath.
	 * 
	 * @return RuleBase class containing the compiled version of the rules ready
	 *         to be loaded into the engine.
	 * @throws Exception
	 *             if any issues occur during the loading of the rules or DSL
	 */
	private RuleBase getRulesBase() throws Exception {
		Reader source = new InputStreamReader(getClass().getResourceAsStream(
				"/Rules.drl"));
		Reader dslSource = new InputStreamReader(getClass()
				.getResourceAsStream("/DSL-Expander.dsl"));
		PackageBuilder builder = new PackageBuilder();
		builder.addPackageFromDrl(source, dslSource);
		Package pkg = builder.getPackage();

		RuleBaseConfiguration conf = new RuleBaseConfiguration();
		conf.setAssertBehaviour(RuleBaseConfiguration.AssertBehaviour.EQUALITY);

		RuleBase ruleBase = RuleBaseFactory.newRuleBase(conf);
		ruleBase.addPackage(pkg);

		return ruleBase;
	}
}
