package com.demo.soa.esb.rulesengine.drools.dao;

import java.util.List;

import com.demo.soa.esb.util.exception.DAOException;
import com.demo.soa.esb.util.exception.XMLException;
import com.demo.soa.esb.util.to.ModelBean;

/**
 * This is a DAO interface that declares all the methods necessary to persist
 * the state of objects in the working memory of the Drools rules engine. The
 * objects are converted to XML before being persisted. This interface is the
 * critical part of the DAO pattern which allows to create multiple
 * implementations of the DAO (JDBC implementation, test implementation, etc).
 * 
 * @author Leon Rubin
 */
public interface WorkingMemoryDAO {
	/**
	 * Reads all persisted working memory objects from the database and converts
	 * them form XML back to Java object form.
	 * 
	 * @return List of objects (facts) that are ready to be inserted in working
	 *         memory
	 * @throws DAOException
	 *             if a problem occurs while querying the data store
	 * @throws XMLException
	 *             if a problem occurs while converting XML to objects
	 */
	List<ModelBean> getAllObjects() throws DAOException, XMLException;

	/**
	 * Writes the state of a new working memory object to the database.
	 * 
	 * @param id
	 *            unique object id
	 * @param xml
	 *            XML representation of the object
	 * @throws DAOException
	 *             if a problem occurs while writing the data to the data store
	 */
	void insert(String id, String xml) throws DAOException;

	/**
	 * Updates the state of an existing working memory object in the database.
	 * 
	 * @param id
	 *            unique object id
	 * @param xml
	 *            XML representation of the object
	 * @throws DAOException
	 *             if a problem occurs while writing the data to the data store
	 */
	void update(String id, String xml) throws DAOException;

	/**
	 * Deletes the object from the data store
	 * 
	 * @param id
	 *            unique object id
	 * @throws DAOException
	 *             if a problem occurs while deleting the object
	 */
	void remove(String id) throws DAOException;

	/**
	 * Releases the resources held by the DAO (database connections, etc).
	 * 
	 * @throws DAOException
	 *             if a problem occurs while releasing resources.
	 */
	void close() throws DAOException;
}
