package com.demo.soa.esb.rulesengine.drools.dao.hsql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import com.demo.soa.esb.rulesengine.drools.dao.WorkingMemoryDAO;
import com.demo.soa.esb.util.exception.DAOException;
import com.demo.soa.esb.util.exception.XMLException;
import com.demo.soa.esb.util.to.ModelBean;
import com.demo.soa.esb.xml.XMLParserFactory;

/**
 * This class is a JDBC/HSQL implementation of the working memory DAO. HSQL is
 * an embedded pure Java database that is portable and has a small footprint.
 * The entire database is contained in a single hsqldb.jar file that is included
 * with this project. This implementation is not intended for production use and
 * was created exclusively for this demo. The database will be created by the
 * application at runtime in the /tmp/DroolsData directory and consists of a
 * handful of files. No other configuration should be necessary.
 * 
 * This JDBC code should not be used as an example of how to implement a DAO
 * since some of the code and the way connections are created and closed is not
 * typical but used to compensate for some quirks of the HSQL database.
 * 
 * Javadoc comments for most of the methods are not provided because they will
 * be inherited from the implemented interface during generation.
 * 
 * @author Leon Rubin
 */
public class WorkingMemoryDAOHSQLImpl implements WorkingMemoryDAO {
	private static final String HSQL_DRIVER_CLASS = "org.hsqldb.jdbcDriver";
	private static final String DATA_PATH = "/tmp/DroolsData/testdb";
	private static final String CONNECTION_STRING = "jdbc:hsqldb:file:"
			+ DATA_PATH + ";shutdown=true";

	private Connection connection;

	public WorkingMemoryDAOHSQLImpl() throws DAOException {
		createFactTable();
	}

	private void createFactTable() throws DAOException {
		connect();

		PreparedStatement ps = null;

		try {
			ps = connection
					.prepareStatement("CREATE CACHED TABLE DROOLS_OBJECT (ID VARCHAR(256), DATA VARCHAR(2048))");

			ps.executeUpdate();
			System.out
					.println("Created a database for storing working memory facts");
		} catch (SQLException e) {
			if (e.getMessage().indexOf("Table already exists") > -1) {
				System.out
						.println("Found a fact database.  Loading facts into working memory");
			} else {
				// Some other issue occured that needs to be thrown to the
				// caller for handling
				throw new DAOException(
						"Failed to create a database for storing working memory facts",
						e);
			}
		} finally {
			close(ps);
		}
	}

	public List<ModelBean> getAllObjects() throws DAOException, XMLException {
		connect();
		PreparedStatement ps = null;
		ResultSet rs = null;
		List<ModelBean> objects = new ArrayList<ModelBean>();

		try {
			ps = connection.prepareStatement("SELECT DATA FROM DROOLS_OBJECT");

			System.out.println("Selecting data from persisted facts table");

			rs = ps.executeQuery();

			while (rs.next()) {
				String xml = rs.getString(1);
				// System.out.println("XML {" + xml + "}");
				objects.add((ModelBean) XMLParserFactory.getXMLParser().parse(
						xml));
			}

			rs.close();

			return objects;
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to load objects", e);
		} finally {
			close(rs);
			close(ps);
		}
	}

	public void insert(String id, String xml) throws DAOException {
		connect();
		PreparedStatement ps = null;

		try {
			ps = connection
					.prepareStatement("INSERT INTO DROOLS_OBJECT VALUES ('"
							+ id + "', '" + xml + "')");

			int rowsAffected = ps.executeUpdate();

			System.out.println("Inserted " + rowsAffected + " row");
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to insert object id=" + id, e);
		} finally {
			close(ps);
		}
	}

	public void remove(String id) throws DAOException {
		connect();
		PreparedStatement ps = null;

		try {
			ps = connection
					.prepareStatement("DELETE FROM DROOLS_OBJECT WHERE ID='"
							+ id + "'");

			ps.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to delete object id=" + id, e);
		} finally {
			close(ps);
		}
	}

	public void update(String id, String xml) throws DAOException {
		connect();
		PreparedStatement ps = null;

		try {
			ps = connection.prepareStatement("UPDATE DROOLS_OBJECT SET DATA='"
					+ xml + "' WHERE ID='" + id + "'");

			ps.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to update object id=" + id, e);
		} finally {
			close(ps);
		}
	}

	private void connect() throws DAOException {
		if (connection == null) {
			try {
				Class.forName(HSQL_DRIVER_CLASS);
				connection = DriverManager.getConnection(CONNECTION_STRING,
						"sa", "");
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				throw new DAOException("Failed to load the driver", e);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new DAOException("Failed to get database connection", e);
			}
		}
	}

	private void close(ResultSet rs) throws DAOException {
		try {
			rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to close result set", e);
		}
	}

	private void close(PreparedStatement ps) throws DAOException {
		try {
			ps.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to close prepared statement", e);
		}
	}

	private void close(Connection connection) throws DAOException {
		try {
			connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new DAOException("Failed to close connection", e);
		}
	}

	public void close() throws DAOException {
		if (connection != null) {
			close(connection);
			connection = null;
		}
	}
}
