package com.demo.soa.esb.util.customj;

import com.demo.soa.esb.util.exception.XMLException;
import com.demo.soa.esb.xml.XMLParser;
import com.demo.soa.esb.xml.XMLParserFactory;
import com.sonicsw.xq.XQEndpoint;
import com.sonicsw.xq.XQEndpointException;
import com.sonicsw.xq.XQEndpointManager;
import com.sonicsw.xq.XQEndpointTimeoutException;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQPart;
import com.sonicsw.xq.XQServiceException;

/**
 * This helper class contains the code necessary to invoke ESB services. Any
 * class that needs to invoke an ESB service should call the <code>send</code>
 * method passing the name of the entry endpoint and an input parameter object.
 * Before sending the message, the caller needs to ensure that the XQ endpoint
 * manager reference is set by calling the <code>setXQEndpointManager</code>
 * method. A good place to initialize the XQ endpoint manager is in the init
 * method of a custom Java service since it is avaialbe from the initial
 * context. The <code>send</code> method will be invoked by the rules inside
 * the rules engine as needed.
 * 
 * @author Leon Rubin
 */
public class EndpointManager {
	private static XQEndpointManager xqEndpointManager;
	private static final int TIMEOUT = 5000;

	/**
	 * Sets the service context that is later used to lookup the endpoint by
	 * name.
	 * 
	 * @param ctx
	 *            service context
	 */
	public static void setXQEndpointManager(XQEndpointManager xqem) {
		xqEndpointManager = xqem;
	}

	/**
	 * Invokes an ESB service by sending a request to an endpoint with the
	 * specified name.
	 * 
	 * @param endpointName
	 *            the name of the entry endpoint of the service that needs to be
	 *            invoked
	 * @param object
	 *            the parameter object that will be converted to XML before
	 *            sending it to the endpoint
	 * @throws XQServiceException
	 *             if a call to the service times out or any other service
	 *             exception occurs
	 * @throws XMLException
	 *             if an error occurs while converting the input object to XML
	 * @throws XQMessageException
	 *             if a message exception occurs
	 */
	public static void send(String endpointName, Object object)
			throws XQServiceException, XMLException, XQMessageException {
		XMLParser xmParser = XMLParserFactory.getXMLParser();

		String xml = xmParser.generate(object);

		try {
			XQEndpoint endpoint = xqEndpointManager.getEndpoint(endpointName);
			XQMessage message = endpoint.createMessage();
			XQPart part = message.createPart();

			part.setContent(xml, "text/xml");
			message.addPart(part);
			endpoint.call(message, TIMEOUT);
		} catch (XQEndpointTimeoutException e) {
			e.printStackTrace();
			throw new XQServiceException("Timeout while calling endpoint "
					+ endpointName, e);
		} catch (XQEndpointException e) {
			e.printStackTrace();
			throw new XQServiceException("Endpoint exception while calling "
					+ endpointName, e);
		}
	}
}
