package com.demo.soa.esb.xml;

import com.demo.soa.esb.util.exception.XMLException;

/**
 * This interface declares the methods that an XML parser needs to implement.
 * This interface is an important part of a factory pattern that allows all
 * classes that use XML processing functions remain independent of the
 * underlying implementation (Castor, DOM, etc). All calling classes must be
 * coded to this interface and obtain an implementation from the factory.
 * 
 * @author Leon Rubin
 */
public interface XMLParser {
	/**
	 * Parses passed XML and builds a Java object that encapsulates the data
	 * contained in the XML. It is assumed that the implementing class will be
	 * configured to determine the type of an object that needs to be created
	 * based on the root element of the XML.
	 * 
	 * @param xml
	 *            String containing the XML to parse
	 * @return Object that represents the data in the XML
	 * @throws XMLException
	 *             if a problem occurs while parsing the XML
	 */
	Object parse(String xml) throws XMLException;

	/**
	 * Generates XML representation of a specified object. It is assumed that
	 * the implementing class will be configured to determine the XML structure
	 * that should be generated based on the type of a specified Java object.
	 * 
	 * @param object
	 *            the object that needs to be represented as XML
	 * @return String containing the XML representation of the specified object
	 * @throws XMLException
	 *             if a problem occurs while parsing the XML
	 */
	String generate(Object object) throws XMLException;

	/**
	 * Releases all resources held by this component.
	 */
	void destroy();
}
