package com.demo.soa.esb.xml.castor;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;

import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.mapping.MappingException;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;
import org.xml.sax.InputSource;

import com.demo.soa.esb.util.exception.XMLException;
import com.demo.soa.esb.xml.XMLParser;

/**
 * This class contains the methods to map Java objects to XML. This class uses
 * the Castor framework to generate the XML (see CastorMapping.xml for mapping
 * configuration details). 
 * 
 * Javadoc comments for most of the methods are not
 * provided because they will be inherited from the implemented interface during
 * generation.
 * 
 * @author Leon Rubin
 */
public class XMLParserCastorImpl implements XMLParser {
	private static final String MAPPING_FILE_NAME = "CastorMapping.xml";

	private static Mapping mapping;
	private static boolean initialized;

	/**
	 * Creates the generator by loading the Castor mapping file.
	 * 
	 * @throws XMLException
	 *             if an error occurs while loading a Castor mapping file
	 */
	private void init() throws XMLException {
		try {
			mapping = new Mapping();

			InputStream inputStream = XMLParserCastorImpl.class
					.getClassLoader().getResourceAsStream(MAPPING_FILE_NAME);

			if (inputStream == null) {
				throw new XMLException("Resource " + MAPPING_FILE_NAME
						+ " not found in classpath");
			}

			mapping.loadMapping(new InputSource(inputStream));

			initialized = true;
		} catch (IOException e) {
			e.printStackTrace();
			throw new XMLException("Failed to create mapping from file "
					+ MAPPING_FILE_NAME, e);
		} catch (MappingException e) {
			e.printStackTrace();
			throw new XMLException("Failed to create mapping from file "
					+ MAPPING_FILE_NAME, e);
		}
	}

	public Object parse(String xml) throws XMLException {
		try {
			if (!initialized) {
				init();
			}

			InputSource inputSource = new InputSource(new StringReader(xml));

			Unmarshaller unmarshaller = new Unmarshaller();

			unmarshaller.setMapping(mapping);
			return unmarshaller.unmarshal(inputSource);
		} catch (MarshalException e) {
			e.printStackTrace();
			throw new XMLException("Failed to unmarshal an object", e);
		} catch (MappingException e) {
			e.printStackTrace();
			throw new XMLException(
					"A mapping error detected while unmarshalling an object", e);
		} catch (ValidationException e) {
			e.printStackTrace();
			throw new XMLException(
					"A validation error detected while unmarshalling an object",
					e);
		}
	}

	public String generate(Object object) throws XMLException {
		try {
			if (!initialized) {
				init();
			}

			StringWriter stringWriter = new StringWriter();
			Marshaller marshaller = new Marshaller(stringWriter);

			marshaller.setMapping(mapping);
			marshaller.marshal(object);

			return stringWriter.toString();
		} catch (MarshalException e) {
			e.printStackTrace();
			throw new XMLException("Failed to marshal an ErrorCollection", e);
		} catch (MappingException e) {
			e.printStackTrace();
			throw new XMLException(
					"A mapping error detected while marshalling an ErrorCollection",
					e);
		} catch (ValidationException e) {
			e.printStackTrace();
			throw new XMLException(
					"A validation error detected while marshalling an ErrorCollection",
					e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new XMLException(
					"An IO error detected while marshalling an ErrorCollection",
					e);
		}
	}

	public void destroy() {

	}
}
